#ifndef ENV_HPP
#define ENV_HPP

#include "temp.hpp"

namespace env {
constexpr int FPARAM_ARRAY_FIRST_DIM = 1;

enum class TyKind {
    VOID,
    INT,
    FLOAT,
    INT_PTR,
    FLOAT_PTR,
};

enum class FuncRetType {
    VOID,
    INT,
    FLOAT,
};

class Type {
public:
    TyKind type;
    bool isConst;

    Type(TyKind type, bool isConst) : type(type), isConst(isConst) {}
};

class GlobalVar {
public:
    Type *type;
    union {
        temp::Label *label;
        int consti;
        float constf;
    } u;
    std::vector<int> *dims;
};
GlobalVar *GlobalConstIntEntry(int consti);
GlobalVar *GlobalConstFloatEntry(float constf);
GlobalVar *GlobalConstIArrEntry(temp::Label *label, std::vector<int> *dims);
GlobalVar *GlobalConstFArrEntry(temp::Label *label, std::vector<int> *dims);
GlobalVar *GlobalIntEntry(temp::Label *label);
GlobalVar *GlobalFloatEntry(temp::Label *label);
GlobalVar *GlobalIArrEntry(temp::Label *label, std::vector<int> *dims);
GlobalVar *GlobalFArrEntry(temp::Label *label, std::vector<int> *dims);

class LocalVar {
public:
    Type *type;
    union {
        temp::Temp *temp;
        temp::Label *label;
        int consti;
        float constf;
    } u;
    std::vector<int> *dims;
};
LocalVar *LocalConstIntEntry(int consti);
LocalVar *LocalConstFloatEntry(float constf);
LocalVar *LocalConstIArrEntry(temp::Label *label, std::vector<int> *dims);
LocalVar *LocalConstFArrEntry(temp::Label *label, std::vector<int> *dims);
LocalVar *LocalIntEntry(temp::Temp *temp);
LocalVar *LocalFloatEntry(temp::Temp *temp);
LocalVar *LocalIArrEntry(temp::Temp *temp, std::vector<int> *dims);
LocalVar *LocalFArrEntry(temp::Temp *temp, std::vector<int> *dims);

class Func {
public:
    FuncRetType retType;
    std::string *name;
    std::vector<TyKind> *paramTypes;
};
Func *FuncEntry(FuncRetType retType, std::string *name, std::vector<TyKind> *paramTypes);

} // namespace env

#endif // ENV_HPP