#ifndef TABLE_HPP
#define TABLE_HPP

#include <deque>
#include <string>
#include <unordered_map>

namespace table {
template <class K, class V> class Table {
protected:
    std::unordered_map<K, V> tab;
    std::deque<std::pair<K, std::pair<V, bool>>> q;

public:
    Table() = default;
    virtual ~Table() = default;

    void enter(K key, V value) {
        if (contains(key)) {
            q.push_back(std::make_pair(key, std::make_pair(tab[key], true)));
        } else {
            q.push_back(std::make_pair(key, std::make_pair(nullptr, false)));
        }
        tab[key] = value;
    }
    bool contains(K key) { return tab.find(key) != tab.end(); }
    V look(K key) {
        if (tab.find(key) != tab.end())
            return tab[key];
        else
            return nullptr;
    }
    std::pair<K, V> pop() {
        auto p = q.back();
        q.pop_back();
        V value = tab[p.first];
        if (p.second.second) {
            tab[p.first] = p.second.first;
        } else {
            tab.erase(p.first);
        }
        return std::make_pair(p.first, value);
    }

    auto begin() -> decltype(q.begin()) { return q.begin(); }
    auto end() -> decltype(q.end()) { return q.end(); }
};

template <class V> class SymbolTable : public Table<std::string, V> {
public:
    SymbolTable() = default;
    ~SymbolTable() = default;

    SymbolTable &operator=(SymbolTable &&rhs) {
        if (this != &rhs) {
            this->tab = std::move(rhs.tab);
            this->q = std::move(rhs.q);
        }
        return *this;
    }

    void beginScope() { this->enter("<mark>", nullptr); }
    void endScope() {
        while (this->q.back().first != "<mark>") {
            this->pop();
        }
        this->pop();
    }
};
} // namespace table

#endif // TABLE_HPP