#ifndef AST_PRINTER_HPP
#define AST_PRINTER_HPP

#include "ast.hpp"
#include <ostream>

namespace ast {
class Printer {
public:
    Printer(std::ostream &os, Prog *root) : os(os), root(root) {}
    void print();
private:
    std::ostream &os;
    Prog *root;
    int indent = 0;

    void printCompUnitList(CompUnitList *node);
    void printCompUnit(CompUnit *node);
    void printDecl(Decl *node);
    void printConstDecl(ConstDecl *node);
    void printConstDefList(ConstDefList *node);
    void printConstDef(ConstDef *node);
    void printArrayDim(ExpList *node);
    void printVarDecl(VarDecl *node);
    void printVarDefList(VarDefList *node);
    void printVarDef(VarDef *node);
    void printInitVal(InitVal *node);
    void printInitValList(InitValList *node);
    void printFuncDef(FuncDef *node);
    void printFuncFParamList(FuncFParamList *node);
    void printFuncFParam(FuncFParam *node);
    void printBlock(Block *node);
    void printBlockItemList(BlockItemList *node);
    void printBlockItem(BlockItem *node);
    void printStmt(Stmt *node);
    void printArgs(ExpList *node);
    void printExp(Exp *node);
    void printBinOp(BinOp op);
    void printRelOp(RelOp op);
};
} // namespace ast

#endif // AST_PRINTER_HPP