#ifndef IRP_PRINTER_HPP
#define IRP_PRINTER_HPP

#include "irp.hpp"
#include <ostream>

namespace irp {
class Printer {
public:
    Printer(std::ostream &os, Prog *root) : os(os), root(root), indent(0) {}
    void print();

private:
    std::ostream &os;
    Prog *root;
    int indent;

    void printGlobalDefs(const std::vector<GlobalDef *> &globalTable);
    void printLocalConstDefs(const std::vector<LocalConstArrayDef *> &localConstArrayTable);
    void printFuncDefList(FuncDefList *node);
    void printFuncDef(FuncDef *node);
    void printTempList(temp::TempList *node);
    void printTemp(temp::Temp *node);

    void printStmt(Stmt *node);
    void printSeqStmt(Stmt *node);
    void printLabelStmt(Stmt *node);
    void printMoveStmt(Stmt *node);
    void printExpStmt(Stmt *node);
    void printReturnStmt(Stmt *node);
    void printJumpStmt(Stmt *node);
    void printCJumpStmt(Stmt *node);
    void printVoidCallStmt(Stmt *node);
    void printPutintStmt(Stmt *node);
    void printPutchStmt(Stmt *node);
    void printPutfloatStmt(Stmt *node);
    void printPutarrayStmt(Stmt *node);
    void printPutfarrayStmt(Stmt *node);
    void printPutfStmt(Stmt *node);
    void printStarttimeStmt();
    void printStoptimeStmt();
    void printMemsetStmt(Stmt *node);

    void printExp(Exp *node);
    void printBinOpExp(Exp *node);
    void printMemExp(Exp *node);
    void printTempExp(Exp *node);
    void printEseqExp(Exp *node);
    void printNameExp(Exp *node);
    void printConstIExp(Exp *node);
    void printConstFExp(Exp *node);
    void printCallExp(Exp *node);
    void printCastExp(Exp *node);
    void printGetintExp();
    void printGetchExp();
    void printGetfloatExp();
    void printGetarrayExp(Exp *node);
    void printGetfarrayExp(Exp *node);
    void printAllocaExp(Exp *node);

    void printExpList(ExpList *node);
};
} // namespace irp

#endif // IRP_PRINTER_HPP