/* 1. declarations */

/* included code */
%{

#include <iostream>
#include <string>
#include "ast.hpp"
#include "parser.hpp"

int c;
int line = 1;
int pos = 1;

// for error reporting
char linebuf[2000];
%}

%option noyywrap

/* start conditions */
%s COMMENT1 COMMENT2

/* regexp nicknames */
format        [\"][^\n]*[\"]
punctuation   [()\[\]{}=,;]
id            [a-z_A-Z][a-z_A-Z0-9]*
integer_8     0[0-7]+
integer_10    [1-9][0-9]*
integer_16    0[xX][0-9a-fA-F]+
float_1       ([0-9]*)\.[0-9]*
float_2       [0-9]+[eE][\+\-]?[0-9]+
float_3       [0-9]*\.[0-9]*[eE][\+\-]?[0-9]+
float_16      0[xX][0-9a-fA-F]*[\.]?[0-9a-fA-F]*[pP][\+\-]?[0-9]+

%% /* 2. rules */

<INITIAL>"//" {
  pos += 2;
  BEGIN COMMENT1;
}
<INITIAL>"/*" {
  pos += 2;
  BEGIN COMMENT2;
}

<COMMENT1>\n {
  line += 1;
  pos = 1;
  BEGIN INITIAL;
}
<COMMENT1>. {
  pos += 1;
}

<COMMENT2>"*/" {
  pos += 2;
  BEGIN INITIAL;
}
<COMMENT2>\n {
  line += 1;
  pos = 1;
}
<COMMENT2>. {
  pos += 1;
}


<INITIAL>" "|\t {
  pos += 1;
}
<INITIAL>\n.* {
  line += 1;
  pos = 1;
  strcpy(linebuf, yytext + 1);
  yyless(1);
}
<INITIAL>\r {}

<INITIAL>{format} {
  yylval.format = new std::string(yytext);
  pos += yyleng;
  return FORMAT;
}

<INITIAL>{punctuation} {
  yylval.pos = ast::make_pos(line, pos);
  pos += 1;
  c = yytext[0];
  return c;
}


<INITIAL>{integer_8} {
  yylval.exp = ast::make_intconst_exp(ast::make_pos(line, pos), (int)strtol(yytext, NULL, 8));
  pos += yyleng;
  return INTCONST;
}
<INITIAL>{integer_10} {
  yylval.exp = ast::make_intconst_exp(ast::make_pos(line, pos), (int)strtol(yytext, NULL, 10));
  pos += yyleng;
  return INTCONST;
}
<INITIAL>{integer_16} {
  yylval.exp = ast::make_intconst_exp(ast::make_pos(line, pos), (int)strtol(yytext, NULL, 16));
  pos += yyleng;
  return INTCONST;
}
<INITIAL>0 {
  yylval.exp = ast::make_intconst_exp(ast::make_pos(line, pos), 0);
  pos += yyleng;
  return INTCONST;
}
<INITIAL>{float_1} {
  yylval.exp = ast::make_floatconst_exp(ast::make_pos(line, pos), new std::string(yytext));
  pos += yyleng;
  return FLOATCONST;
}
<INITIAL>{float_2} {
  yylval.exp = ast::make_floatconst_exp(ast::make_pos(line, pos), new std::string(yytext));
  pos += yyleng;
  return FLOATCONST;
}
<INITIAL>{float_3} {
  yylval.exp = ast::make_floatconst_exp(ast::make_pos(line, pos), new std::string(yytext));
  pos += yyleng;
  return FLOATCONST;
}
<INITIAL>{float_16} {
  yylval.exp = ast::make_floatconst_exp(ast::make_pos(line, pos), new std::string(yytext));
  pos += yyleng;
  return FLOATCONST;
}

<INITIAL>"const" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return CONST;
}
<INITIAL>"int" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return INT;
}
<INITIAL>"float" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return FLOAT;
}
<INITIAL>"void" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return VOID;
}
<INITIAL>"if" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return IF;
}
<INITIAL>"else" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return ELSE;
}
<INITIAL>"while" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return WHILE;
}
<INITIAL>"continue" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return CONTINUE;
}
<INITIAL>"break" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return BREAK;
}
<INITIAL>"return" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return RETURN;
}
<INITIAL>"putint" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return PUTINT;
}
<INITIAL>"putch" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return PUTCH;
}
<INITIAL>"putfloat" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return PUTFLOAT;
}
<INITIAL>"putarray" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return PUTARRAY;
}
<INITIAL>"putfarray" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return PUTFARRAY;
}
<INITIAL>"putf" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return PUTF;
}
<INITIAL>"starttime" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return STARTTIME;
}
<INITIAL>"stoptime" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return STOPTIME;
}
<INITIAL>"getint" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return GETINT;
}
<INITIAL>"getch" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return GETCH;
}
<INITIAL>"getfloat" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return GETFLOAT;
}
<INITIAL>"getarray" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return GETARRAY;
}
<INITIAL>"getfarray" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return GETFARRAY;
}


<INITIAL>"+" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return ADD;
}
<INITIAL>"-" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return SUB;
}
<INITIAL>"*" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return MUL;
}
<INITIAL>"/" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return DIV;
}
<INITIAL>"%" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return MOD;
}
<INITIAL>"||" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return OR;
}
<INITIAL>"&&" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return AND;
}
<INITIAL>"!" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return NOT;
}
<INITIAL>"==" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return EQ;
}
<INITIAL>"!=" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return NE;
}
<INITIAL>"<" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return LT;
}
<INITIAL>"<=" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return LE;
}
<INITIAL>">" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return GT;
}
<INITIAL>">=" {
  yylval.pos = ast::make_pos(line, pos);
  pos += yyleng;
  return GE;
}

<INITIAL>{id} {
  yylval.exp = ast::make_id_exp(ast::make_pos(line, pos), new std::string(yytext));
  pos += yyleng;
  return ID;
}

<INITIAL>. {
  printf("Illegal input \"%c\" at line %d, position %d\n", yytext[0], line, pos);
  pos++;
}

%% /* 3. programs */