/* 1. declarations */

/* included code */
%{
#include <iostream>
#include <string>
#include "ast.hpp"

extern ast::Prog *root;
extern int yylex();
extern void yyerror(const char *);
extern int yywrap();
%}

/* yylval */
%union {
  ast::Pos *pos;
  ast::Prog *prog;
  ast::CompUnitList *compUnitList;
  ast::CompUnit *compUnit;
  ast::Decl *decl;
  ast::ConstDecl *constDecl;
  ast::ConstDefList *constDefList;
  ast::ConstDef *constDef;
  ast::InitVal *initVal;
  ast::InitValList *initValList;
  ast::VarDecl *varDecl;
  ast::VarDefList *varDefList;
  ast::VarDef *varDef;
  ast::FuncDef *funcDef;
  ast::FuncFParamList *funcFParamList;
  ast::FuncFParam *funcFParam;
  ast::Block *block;
  ast::BlockItemList *blockItemList;
  ast::BlockItem *blockItem;
  ast::Stmt *stmt;
  ast::ExpList *expList;
  ast::Exp *exp;
  std::string *format;
}

/* termianl symbols */
// reserved keywords
%token<pos> CONST
%token<pos> INT FLOAT VOID
%token<pos> IF ELSE
%token<pos> WHILE CONTINUE BREAK
%token<pos> RETURN

// functions
%token<pos> PUTINT PUTCH PUTFLOAT PUTARRAY PUTFARRAY PUTF
%token<pos> STARTTIME STOPTIME
%token<pos> GETINT GETCH GETFLOAT GETARRAY GETFARRAY

// operators
%token<pos> ADD SUB MUL DIV MOD
%token<pos> OR AND NOT
%token<pos> EQ NE LT LE GT GE

// identifiers
%token<exp> ID

// num
%token<exp> INTCONST FLOATCONST

// punctuation
%token<pos> '(' ')' '[' ']' '{' '}' '=' ',' ';'

// format string for putf
%token<format> FORMAT

/* non-terminal symbols */
%type<prog> PROG
%type<compUnitList> COMP_UNIT_LIST
%type<compUnit> COMP_UNIT
%type<decl> DECL
%type<constDecl> CONST_DECL
%type<constDefList> CONST_DEF_REST
%type<constDef> CONST_DEF
%type<initVal> INIT_VAL
%type<initValList> INIT_VAL_LIST INIT_VAL_REST
%type<varDecl> VAR_DECL
%type<varDefList> VAR_DEF_REST
%type<varDef> VAR_DEF
%type<funcDef> FUNC_DEF
%type<funcFParamList> FUNC_FPARAM_LIST FUNC_FPARAM_REST
%type<funcFParam> FUNC_FPARAM
%type<block> BLOCK
%type<blockItemList> BLOCK_ITEM_LIST
%type<blockItem> BLOCK_ITEM
%type<stmt> STMT
%type<expList> ARRAY_DIM FUNC_RPARAM_LIST FUNC_RPARAM_REST
%type<exp> EXP ADD_EXP MUL_EXP REL_EXP EQ_EXP LOR_EXP LAND_EXP LVAL NUMBER COND PRIMARY_EXP UNARY_EXP
%type<exp> ID_ALLOWED

%start PROG

%precedence THEN
%precedence ELSE

%% /* 2. rules */

PROG: COMP_UNIT_LIST {
  $$ = new ast::Prog($1);
  root = $$;
}

COMP_UNIT_LIST: COMP_UNIT {
  $$ = ast::make_comp_unit_list($1, nullptr);
} | COMP_UNIT COMP_UNIT_LIST {
  $$ = ast::make_comp_unit_list($1, $2);
}

COMP_UNIT: DECL {
  $$ = ast::make_unit_decl($1->pos, $1);
} | FUNC_DEF {
  $$ = ast::make_unit_func($1->pos, $1);
}

DECL: CONST_DECL {
  $$ = ast::make_decl_const($1->pos, $1);
} | VAR_DECL {
  $$ = ast::make_decl_var($1->pos, $1);
}

CONST_DECL: CONST INT CONST_DEF CONST_DEF_REST ';' {
  $$ = ast::make_const_decl($1, ast::BaseType::INT, ast::make_const_def_list($3, $4));
} | CONST FLOAT CONST_DEF CONST_DEF_REST ';' {
  $$ = ast::make_const_decl($1, ast::BaseType::FLOAT, ast::make_const_def_list($3, $4));
}

CONST_DEF_REST: /* empty */ {
  $$ = nullptr;
} | ',' CONST_DEF CONST_DEF_REST {
  $$ = ast::make_const_def_list($2, $3);
}

CONST_DEF: ID_ALLOWED ARRAY_DIM '=' INIT_VAL {
  $$ = ast::make_const_def($1->pos, $1->u.id, $2, $4);
}

ARRAY_DIM: /* empty */ {
  $$ = nullptr;
} | '[' EXP ']' ARRAY_DIM {
  $$ = ast::make_exp_list($2, $4);
}

INIT_VAL: EXP {
  $$ = ast::make_init_val_exp($1->pos, $1);
} | '{' INIT_VAL_LIST '}' {
  $$ = ast::make_init_val_array($1, $2);
}

INIT_VAL_LIST: /* empty */ {
  $$ = nullptr;
} | INIT_VAL INIT_VAL_REST {
  $$ = ast::make_init_val_list($1, $2);
}

INIT_VAL_REST: /* empty */ {
  $$ = nullptr;
} | ',' INIT_VAL INIT_VAL_REST {
  $$ = ast::make_init_val_list($2, $3);
}

VAR_DECL: INT VAR_DEF VAR_DEF_REST ';' {
  $$ = ast::make_var_decl($1, ast::BaseType::INT, ast::make_var_def_list($2, $3));
} | FLOAT VAR_DEF VAR_DEF_REST ';' {
  $$ = ast::make_var_decl($1, ast::BaseType::FLOAT, ast::make_var_def_list($2, $3));
}

VAR_DEF_REST: /* empty */ {
  $$ = nullptr;
} | ',' VAR_DEF VAR_DEF_REST {
  $$ = ast::make_var_def_list($2, $3);
}

VAR_DEF: ID_ALLOWED ARRAY_DIM {
  $$ = ast::make_var_def($1->pos, $1->u.id, $2, nullptr);
} | ID_ALLOWED ARRAY_DIM '=' INIT_VAL {
  $$ = ast::make_var_def($1->pos, $1->u.id, $2, $4);
}

FUNC_DEF: VOID ID '(' FUNC_FPARAM_LIST ')' BLOCK {
  $$ = ast::make_func_def($1, ast::FuncType::VOID, $2->u.id, $4, $6);
} | INT ID '(' FUNC_FPARAM_LIST ')' BLOCK {
  $$ = ast::make_func_def($1, ast::FuncType::INT, $2->u.id, $4, $6);
} | FLOAT ID '(' FUNC_FPARAM_LIST ')' BLOCK {
  $$ = ast::make_func_def($1, ast::FuncType::FLOAT, $2->u.id, $4, $6);
}

FUNC_FPARAM_LIST: /* empty */ {
  $$ = nullptr;
} | FUNC_FPARAM FUNC_FPARAM_REST {
  $$ = ast::make_func_f_param_list($1, $2);
}

FUNC_FPARAM_REST: /* empty */ {
  $$ = nullptr;
} | ',' FUNC_FPARAM FUNC_FPARAM_REST {
  $$ = ast::make_func_f_param_list($2, $3);
}

FUNC_FPARAM: INT ID_ALLOWED {
  $$ = ast::make_func_f_param($1, ast::BaseType::INT, $2->u.id, false, nullptr);
} | FLOAT ID_ALLOWED {
  $$ = ast::make_func_f_param($1, ast::BaseType::FLOAT, $2->u.id, false, nullptr);
} | INT ID_ALLOWED '[' ']' ARRAY_DIM {
  $$ = ast::make_func_f_param($1, ast::BaseType::INT, $2->u.id, true, $5);
} | FLOAT ID_ALLOWED '[' ']' ARRAY_DIM {
  $$ = ast::make_func_f_param($1, ast::BaseType::FLOAT, $2->u.id, true, $5);
}

BLOCK: '{' BLOCK_ITEM_LIST '}' {
  $$ = ast::make_block($1, $2);
}

BLOCK_ITEM_LIST: /* empty */ {
  $$ = nullptr;
} | BLOCK_ITEM BLOCK_ITEM_LIST {
  $$ = ast::make_block_item_list($1, $2);
}

BLOCK_ITEM: DECL {
  $$ = ast::make_block_item_decl($1->pos, $1);
} | STMT {
  $$ = ast::make_block_item_stmt($1->pos, $1);
}

STMT: LVAL '=' EXP ';' {
  $$ = ast::make_assign_stmt($1->pos, $1, $3);
} | ';' {
  $$ = ast::make_exp_stmt($1, nullptr);
} | EXP ';' {
  $$ = ast::make_exp_stmt($1->pos, $1);
} | BLOCK {
  $$ = ast::make_block_stmt($1->pos, $1);
} | IF '(' COND ')' STMT ELSE STMT {
  $$ = ast::make_if_stmt($1, $3, $5, $7);
} | IF '(' COND ')' STMT %prec THEN {
  $$ = ast::make_if_stmt($1, $3, $5, nullptr);
} | WHILE '(' COND ')' STMT {
  $$ = ast::make_while_stmt($1, $3, $5);
} | CONTINUE ';' {
  $$ = ast::make_continue_stmt($1);
} | BREAK ';' {
  $$ = ast::make_break_stmt($1);
} | RETURN ';' {
  $$ = ast::make_return_stmt($1, nullptr);
} | RETURN EXP ';' {
  $$ = ast::make_return_stmt($1, $2);
} | PUTINT '(' EXP ')' ';' {
  $$ = ast::make_putint_stmt($1, $3);
} | PUTCH '(' EXP ')' ';' {
  $$ = ast::make_putch_stmt($1, $3);
} | PUTFLOAT '(' EXP ')' ';' {
  $$ = ast::make_putfloat_stmt($1, $3);
} | PUTARRAY '(' EXP ',' EXP ')' ';' {
  $$ = ast::make_putarray_stmt($1, $3, $5);
} | PUTFARRAY '(' EXP ',' EXP ')' ';' {
  $$ = ast::make_putfarray_stmt($1, $3, $5);
} | PUTF '(' FORMAT ',' FUNC_RPARAM_LIST ')' ';' {
  $$ = ast::make_putf_stmt($1, $3, $5);
} | STARTTIME '(' ')' ';' {
  $$ = ast::make_starttime_stmt($1);
} | STOPTIME '(' ')' ';' {
  $$ = ast::make_stoptime_stmt($1);
}

EXP: ADD_EXP {
  $$ = $1;
}

COND: LOR_EXP {
  $$ = $1;
}

LVAL: ID_ALLOWED {
  $$ = $1;
} | ID_ALLOWED '[' EXP ']' ARRAY_DIM {
  $$ = ast::make_array_exp($1->pos, $1, ast::make_exp_list($3, $5));
}

PRIMARY_EXP: '(' EXP ')' {
  $2->pos = $1;
  $$ = $2;
} | LVAL {
  $$ = $1;
} | NUMBER {
  $$ = $1;
}

NUMBER: INTCONST {
  $$ = $1;
} | FLOATCONST {
  $$ = $1;
}

UNARY_EXP: PRIMARY_EXP {
  $$ = $1;
} | ID '(' FUNC_RPARAM_LIST ')' {
  $$ = ast::make_call_exp($1->pos, $1->u.id, $3);
} | GETINT '(' ')' {
  $$ = ast::make_getint_exp($1);
} | GETCH '(' ')' {
  $$ = ast::make_getch_exp($1);
} | GETFLOAT '(' ')' {
  $$ = ast::make_getfloat_exp($1);
} | GETARRAY '(' EXP ')' {
  $$ = ast::make_getarray_exp($1, $3);
} | GETFARRAY '(' EXP ')' {
  $$ = ast::make_getfarray_exp($1, $3);
} | ADD UNARY_EXP {
  $2->pos = $1;
  $$ = $2;
} | SUB UNARY_EXP {
  $$ = ast::make_minus_exp($1, $2);
} | NOT UNARY_EXP {
  $$ = ast::make_not_exp($1, $2);
}

FUNC_RPARAM_LIST: /* empty */ {
  $$ = nullptr;
} | EXP FUNC_RPARAM_REST {
  $$ = ast::make_exp_list($1, $2);
}

FUNC_RPARAM_REST: /* empty */ {
  $$ = nullptr;
} | ',' EXP FUNC_RPARAM_REST {
  $$ = ast::make_exp_list($2, $3);
}

MUL_EXP: UNARY_EXP {
  $$ = $1;
} | MUL_EXP MUL UNARY_EXP {
  $$ = ast::make_binop_exp($1->pos, ast::BinOp::MUL, $1, $3);
} | MUL_EXP DIV UNARY_EXP {
  $$ = ast::make_binop_exp($1->pos, ast::BinOp::DIV, $1, $3);
} | MUL_EXP MOD UNARY_EXP {
  $$ = ast::make_binop_exp($1->pos, ast::BinOp::MOD, $1, $3);
}

ADD_EXP: MUL_EXP {
  $$ = $1;
} | ADD_EXP ADD MUL_EXP {
  $$ = ast::make_binop_exp($1->pos, ast::BinOp::ADD, $1, $3);
} | ADD_EXP SUB MUL_EXP {
  $$ = ast::make_binop_exp($1->pos, ast::BinOp::SUB, $1, $3);
}

REL_EXP: ADD_EXP {
  $$ = $1;
} | REL_EXP LT ADD_EXP {
  $$ = ast::make_cond_exp($1->pos, ast::RelOp::LT, $1, $3);
} | REL_EXP GT ADD_EXP {
  $$ = ast::make_cond_exp($1->pos, ast::RelOp::GT, $1, $3);
} | REL_EXP LE ADD_EXP {
  $$ = ast::make_cond_exp($1->pos, ast::RelOp::LE, $1, $3);
} | REL_EXP GE ADD_EXP {
  $$ = ast::make_cond_exp($1->pos, ast::RelOp::GE, $1, $3);
}

EQ_EXP: REL_EXP {
  $$ = $1;
} | EQ_EXP EQ REL_EXP {
  $$ = ast::make_cond_exp($1->pos, ast::RelOp::EQ, $1, $3);
} | EQ_EXP NE REL_EXP {
  $$ = ast::make_cond_exp($1->pos, ast::RelOp::NE, $1, $3);
}

LAND_EXP: EQ_EXP {
  $$ = $1;
} | LAND_EXP AND EQ_EXP {
  $$ = ast::make_cond_exp($1->pos, ast::RelOp::AND, $1, $3);
}

LOR_EXP: LAND_EXP {
  $$ = $1;
} | LOR_EXP OR LAND_EXP {
  $$ = ast::make_cond_exp($1->pos, ast::RelOp::OR, $1, $3);
}

ID_ALLOWED: ID {
  $$ = $1;
} | PUTINT {
  $$ = ast::make_id_exp($1, new std::string("putint"));
} | PUTCH {
  $$ = ast::make_id_exp($1, new std::string("putch"));
} | PUTFLOAT {
  $$ = ast::make_id_exp($1, new std::string("putfloat"));
} | PUTARRAY {
  $$ = ast::make_id_exp($1, new std::string("putarray"));
} | PUTFARRAY {
  $$ = ast::make_id_exp($1, new std::string("putfarray"));
} | PUTF {
  $$ = ast::make_id_exp($1, new std::string("putf"));
} | STARTTIME {
  $$ = ast::make_id_exp($1, new std::string("starttime"));
} | STOPTIME {
  $$ = ast::make_id_exp($1, new std::string("stoptime"));
} | GETINT {
  $$ = ast::make_id_exp($1, new std::string("getint"));
} | GETCH {
  $$ = ast::make_id_exp($1, new std::string("getch"));
} | GETFLOAT {
  $$ = ast::make_id_exp($1, new std::string("getfloat"));
} | GETARRAY {
  $$ = ast::make_id_exp($1, new std::string("getarray"));
} | GETFARRAY {
  $$ = ast::make_id_exp($1, new std::string("getfarray"));
}

%% /* 3. programs */

void yyerror(const char *s) {
  std::cerr << s << std::endl;
}

int yywrap() {
  return 1;
}