#include "ast.hpp"

namespace ast {
Pos *make_pos(int line, int pos) { return new Pos(line, pos); }

Prog *make_prog(CompUnitList *comp_unit_list) { return new Prog(comp_unit_list); }

CompUnitList *make_comp_unit_list(CompUnit *comp_unit, CompUnitList *comp_unit_list) {
    if (comp_unit_list) {
        comp_unit_list->push_front(comp_unit);
        return comp_unit_list;
    } else {
        auto new_comp_unit_list = new CompUnitList();
        new_comp_unit_list->push_back(comp_unit);
        return new_comp_unit_list;
    }
}

CompUnit *make_unit_decl(Pos *pos, Decl *decl) {
    auto comp_unit = new CompUnit();
    comp_unit->pos = pos;
    comp_unit->kind = CompUnitKind::DECL;
    comp_unit->u.decl = decl;
    return comp_unit;
}

CompUnit *make_unit_func(Pos *pos, FuncDef *func_def) {
    auto comp_unit = new CompUnit();
    comp_unit->pos = pos;
    comp_unit->kind = CompUnitKind::FUNC_DEF;
    comp_unit->u.func_def = func_def;
    return comp_unit;
}

Decl *make_decl_const(Pos *pos, ConstDecl *const_decl) {
    auto decl = new Decl();
    decl->pos = pos;
    decl->kind = DeclKind::CONST;
    decl->u.const_decl = const_decl;
    return decl;
}

Decl *make_decl_var(Pos *pos, VarDecl *var_decl) {
    auto decl = new Decl();
    decl->pos = pos;
    decl->kind = DeclKind::VAR;
    decl->u.var_decl = var_decl;
    return decl;
}

ConstDecl *make_const_decl(Pos *pos, BaseType type, ConstDefList *const_defs) {
    auto const_decl = new ConstDecl();
    const_decl->pos = pos;
    const_decl->type = type;
    const_decl->const_defs = const_defs;
    return const_decl;
}

ConstDefList *make_const_def_list(ConstDef *const_def, ConstDefList *const_def_list) {
    if (const_def_list) {
        const_def_list->push_front(const_def);
        return const_def_list;
    } else {
        auto new_const_def_list = new ConstDefList();
        new_const_def_list->push_back(const_def);
        return new_const_def_list;
    }
}

ConstDef *make_const_def(Pos *pos, std::string *id, ExpList *array_dim, InitVal *init_val) {
    auto const_def = new ConstDef();
    const_def->pos = pos;
    const_def->id = id;
    const_def->array_dim = array_dim;
    const_def->init_val = init_val;
    return const_def;
}

InitVal *make_init_val_exp(Pos *pos, Exp *val) {
    auto init_val = new InitVal();
    init_val->pos = pos;
    init_val->kind = InitValKind::VAL;
    init_val->u.val = val;
    return init_val;
}

InitVal *make_init_val_array(Pos *pos, InitValList *array) {
    auto init_val = new InitVal();
    init_val->pos = pos;
    init_val->kind = InitValKind::ARRAY;
    init_val->u.array = array;
    return init_val;
}

InitValList *make_init_val_list(InitVal *init_val, InitValList *init_val_list) {
    if (init_val_list) {
        init_val_list->push_front(init_val);
        return init_val_list;
    } else {
        auto new_init_val_list = new InitValList();
        new_init_val_list->push_back(init_val);
        return new_init_val_list;
    }
}

VarDecl *make_var_decl(Pos *pos, BaseType type, VarDefList *var_defs) {
    auto var_decl = new VarDecl();
    var_decl->pos = pos;
    var_decl->type = type;
    var_decl->var_defs = var_defs;
    return var_decl;
}

VarDefList *make_var_def_list(VarDef *var_def, VarDefList *var_def_list) {
    if (var_def_list) {
        var_def_list->push_front(var_def);
        return var_def_list;
    } else {
        auto new_var_def_list = new VarDefList();
        new_var_def_list->push_back(var_def);
        return new_var_def_list;
    }
}

VarDef *make_var_def(Pos *pos, std::string *id, ExpList *array_dim, InitVal *init_val) {
    auto var_def = new VarDef();
    var_def->pos = pos;
    var_def->id = id;
    var_def->array_dim = array_dim;
    var_def->init_val = init_val;
    return var_def;
}

FuncDef *make_func_def(Pos *pos, FuncType ret_type, std::string *id, FuncFParamList *f_param_list, Block *block) {
    auto func_def = new FuncDef();
    func_def->pos = pos;
    func_def->ret_type = ret_type;
    func_def->id = id;
    func_def->f_param_list = f_param_list;
    func_def->block = block;
    return func_def;
}

FuncFParamList *make_func_f_param_list(FuncFParam *f_param, FuncFParamList *f_param_list) {
    if (f_param_list) {
        f_param_list->push_front(f_param);
        return f_param_list;
    } else {
        auto new_f_param_list = new FuncFParamList();
        new_f_param_list->push_back(f_param);
        return new_f_param_list;
    }
}

FuncFParam *make_func_f_param(Pos *pos, BaseType type, std::string *id, bool is_array, ExpList *array_dim) {
    auto f_param = new FuncFParam();
    f_param->pos = pos;
    f_param->type = type;
    f_param->id = id;
    f_param->is_array = is_array;
    f_param->array_dim = array_dim;
    return f_param;
}

Block *make_block(Pos *pos, BlockItemList *block_item_list) {
    auto block = new Block();
    block->pos = pos;
    block->block_item_list = block_item_list;
    return block;
}

BlockItemList *make_block_item_list(BlockItem *block_item, BlockItemList *block_item_list) {
    if (block_item_list) {
        block_item_list->push_front(block_item);
        return block_item_list;
    } else {
        auto new_block_item_list = new BlockItemList();
        new_block_item_list->push_back(block_item);
        return new_block_item_list;
    }
}

BlockItem *make_block_item_decl(Pos *pos, Decl *decl) {
    auto block_item = new BlockItem();
    block_item->pos = pos;
    block_item->kind = BlockItemKind::DECL;
    block_item->u.decl = decl;
    return block_item;
}

BlockItem *make_block_item_stmt(Pos *pos, Stmt *stmt) {
    auto block_item = new BlockItem();
    block_item->pos = pos;
    block_item->kind = BlockItemKind::STMT;
    block_item->u.stmt = stmt;
    return block_item;
}

Stmt *make_assign_stmt(Pos *pos, Exp *lhs, Exp *rhs) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::ASSIGN;
    stmt->u.assign.lhs = lhs;
    stmt->u.assign.rhs = rhs;
    return stmt;
}

Stmt *make_exp_stmt(Pos *pos, Exp *exp) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::EXP;
    stmt->u.exp = exp;
    return stmt;
}

Stmt *make_block_stmt(Pos *pos, Block *block) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::BLOCK;
    stmt->u.block = block;
    return stmt;
}

Stmt *make_if_stmt(Pos *pos, Exp *cond, Stmt *thenStmt, Stmt *elseStmt) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::IF;
    stmt->u.if_stmt.cond = cond;
    stmt->u.if_stmt.thenStmt = thenStmt;
    stmt->u.if_stmt.elseStmt = elseStmt;
    return stmt;
}

Stmt *make_while_stmt(Pos *pos, Exp *cond, Stmt *body) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::WHILE;
    stmt->u.while_stmt.cond = cond;
    stmt->u.while_stmt.body = body;
    return stmt;
}

Stmt *make_break_stmt(Pos *pos) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::BREAK;
    return stmt;
}

Stmt *make_continue_stmt(Pos *pos) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::CONTINUE;
    return stmt;
}

Stmt *make_return_stmt(Pos *pos, Exp *exp) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::RETURN;
    stmt->u.exp = exp;
    return stmt;
}

Stmt *make_putint_stmt(Pos *pos, Exp *exp) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::PUTINT;
    stmt->u.exp = exp;
    return stmt;
}

Stmt *make_putch_stmt(Pos *pos, Exp *exp) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::PUTCH;
    stmt->u.exp = exp;
    return stmt;
}

Stmt *make_putfloat_stmt(Pos *pos, Exp *exp) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::PUTFLOAT;
    stmt->u.exp = exp;
    return stmt;
}

Stmt *make_putarray_stmt(Pos *pos, Exp *array, Exp *index) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::PUTARRAY;
    stmt->u.putarray.array = array;
    stmt->u.putarray.out_len = index;
    return stmt;
}

Stmt *make_putfarray_stmt(Pos *pos, Exp *array, Exp *index) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::PUTFARRAY;
    stmt->u.putarray.array = array;
    stmt->u.putarray.out_len = index;
    return stmt;
}

Stmt *make_putf_stmt(Pos *pos, std::string *fmtPtr, ExpList *args) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::PUTF;
    stmt->u.putf.fmt = fmtPtr;
    stmt->u.putf.args = args;
    return stmt;
}

Stmt *make_starttime_stmt(Pos *pos) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::STARTTIME;
    return stmt;
}

Stmt *make_stoptime_stmt(Pos *pos) {
    auto stmt = new Stmt();
    stmt->pos = pos;
    stmt->kind = StmtKind::STOPTIME;
    return stmt;
}

ExpList *make_exp_list(Exp *exp, ExpList *exp_list) {
    if (exp_list) {
        exp_list->push_front(exp);
        return exp_list;
    } else {
        auto new_exp_list = new ExpList();
        new_exp_list->push_back(exp);
        return new_exp_list;
    }
}

Exp *make_binop_exp(Pos *pos, BinOp op, Exp *lhs, Exp *rhs) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::BINOP;
    exp->u.binop.op = op;
    exp->u.binop.lhs = lhs;
    exp->u.binop.rhs = rhs;
    return exp;
}

Exp *make_cond_exp(Pos *pos, RelOp op, Exp *lhs, Exp *rhs) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::COND;
    exp->u.cond.op = op;
    exp->u.cond.lhs = lhs;
    exp->u.cond.rhs = rhs;
    return exp;
}

Exp *make_id_exp(Pos *pos, std::string *id) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::ID;
    exp->u.id = id;
    return exp;
}

Exp *make_intconst_exp(Pos *pos, int int_val) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::INTCONST;
    exp->u.int_val = int_val;
    return exp;
}

Exp *make_floatconst_exp(Pos *pos, std::string *float_val) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::FLOATCONST;
    exp->u.float_val = float_val;
    return exp;
}

Exp *make_not_exp(Pos *pos, Exp *exp) {
    auto new_exp = new Exp();
    new_exp->pos = pos;
    new_exp->kind = ExpKind::NOT;
    new_exp->u.exp = exp;
    return new_exp;
}

Exp *make_minus_exp(Pos *pos, Exp *exp) {
    auto new_exp = new Exp();
    new_exp->pos = pos;
    new_exp->kind = ExpKind::MINUS;
    new_exp->u.exp = exp;
    return new_exp;
}

Exp *make_call_exp(Pos *pos, std::string *funcname, ExpList *args) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::CALL;
    exp->u.call.funcname = funcname;
    exp->u.call.args = args;
    return exp;
}

Exp *make_array_exp(Pos *pos, Exp *array, ExpList *index) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::ARRAY;
    exp->u.array.array = array;
    exp->u.array.index = index;
    return exp;
}

Exp *make_getint_exp(Pos *pos) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::GETINT;
    return exp;
}

Exp *make_getch_exp(Pos *pos) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::GETCH;
    return exp;
}

Exp *make_getfloat_exp(Pos *pos) {
    auto exp = new Exp();
    exp->pos = pos;
    exp->kind = ExpKind::GETFLOAT;
    return exp;
}

Exp *make_getarray_exp(Pos *pos, Exp *exp) {
    auto new_exp = new Exp();
    new_exp->pos = pos;
    new_exp->kind = ExpKind::GETARRAY;
    new_exp->u.exp = exp;
    return new_exp;
}

Exp *make_getfarray_exp(Pos *pos, Exp *exp) {
    auto new_exp = new Exp();
    new_exp->pos = pos;
    new_exp->kind = ExpKind::GETFARRAY;
    new_exp->u.exp = exp;
    return new_exp;
}

} // namespace ast