#include "env.hpp"

namespace env {
GlobalVar *GlobalConstIntEntry(int consti) {
    GlobalVar *var = new GlobalVar();
    var->type = new Type(TyKind::INT, true);
    var->u.consti = consti;
    return var;
}

GlobalVar *GlobalConstFloatEntry(float constf) {
    GlobalVar *var = new GlobalVar();
    var->type = new Type(TyKind::FLOAT, true);
    var->u.constf = constf;
    return var;
}

GlobalVar *GlobalConstIArrEntry(temp::Label *label, std::vector<int> *dims) {
    GlobalVar *var = new GlobalVar();
    var->type = new Type(TyKind::INT_PTR, true);
    var->u.label = label;
    var->dims = dims;
    return var;
}

GlobalVar *GlobalConstFArrEntry(temp::Label *label, std::vector<int> *dims) {
    GlobalVar *var = new GlobalVar();
    var->type = new Type(TyKind::FLOAT_PTR, true);
    var->u.label = label;
    var->dims = dims;
    return var;
}

GlobalVar *GlobalIntEntry(temp::Label *label) {
    GlobalVar *var = new GlobalVar();
    var->type = new Type(TyKind::INT, false);
    var->u.label = label;
    return var;
}

GlobalVar *GlobalFloatEntry(temp::Label *label) {
    GlobalVar *var = new GlobalVar();
    var->type = new Type(TyKind::FLOAT, false);
    var->u.label = label;
    return var;
}

GlobalVar *GlobalIArrEntry(temp::Label *label, std::vector<int> *dims) {
    GlobalVar *var = new GlobalVar();
    var->type = new Type(TyKind::INT_PTR, false);
    var->u.label = label;
    var->dims = dims;
    return var;
}

GlobalVar *GlobalFArrEntry(temp::Label *label, std::vector<int> *dims) {
    GlobalVar *var = new GlobalVar();
    var->type = new Type(TyKind::FLOAT_PTR, false);
    var->u.label = label;
    var->dims = dims;
    return var;
}

LocalVar *LocalConstIntEntry(int consti) {
    LocalVar *var = new LocalVar();
    var->type = new Type(TyKind::INT, true);
    var->u.consti = consti;
    return var;
}

LocalVar *LocalConstFloatEntry(float constf) {
    LocalVar *var = new LocalVar();
    var->type = new Type(TyKind::FLOAT, true);
    var->u.constf = constf;
    return var;
}

LocalVar *LocalConstIArrEntry(temp::Label *label, std::vector<int> *dims) {
    LocalVar *var = new LocalVar();
    var->type = new Type(TyKind::INT_PTR, true);
    var->u.label = label;
    var->dims = dims;
    return var;
}

LocalVar *LocalConstFArrEntry(temp::Label *label, std::vector<int> *dims) {
    LocalVar *var = new LocalVar();
    var->type = new Type(TyKind::FLOAT_PTR, true);
    var->u.label = label;
    var->dims = dims;
    return var;
}

LocalVar *LocalIntEntry(temp::Temp *temp) {
    LocalVar *var = new LocalVar();
    var->type = new Type(TyKind::INT, false);
    var->u.temp = temp;
    return var;
}

LocalVar *LocalFloatEntry(temp::Temp *temp) {
    LocalVar *var = new LocalVar();
    var->type = new Type(TyKind::FLOAT, false);
    var->u.temp = temp;
    return var;
}

LocalVar *LocalIArrEntry(temp::Temp *temp, std::vector<int> *dims) {
    LocalVar *var = new LocalVar();
    var->type = new Type(TyKind::INT_PTR, false);
    var->u.temp = temp;
    var->dims = dims;
    return var;
}

LocalVar *LocalFArrEntry(temp::Temp *temp, std::vector<int> *dims) {
    LocalVar *var = new LocalVar();
    var->type = new Type(TyKind::FLOAT_PTR, false);
    var->u.temp = temp;
    var->dims = dims;
    return var;
}

Func *FuncEntry(FuncRetType retType, std::string *name, std::vector<TyKind> *paramTypes) {
    Func *func = new Func();
    func->retType = retType;
    func->name = name;
    func->paramTypes = paramTypes;
    return func;
}
} // namespace env