#include "ast.hpp"
#include "ast2irp.hpp"
#include "astPrinter.hpp"
#include "canon.hpp"
#include "irpPrinter.hpp"
#include "stmPrinter.hpp"

#include <cstdio>
#include <fstream>
#include <iostream>

extern int yyparse();
ast::Prog *root;

int main(int argc, const char *argv[]) {
    /* parse the command line arguments */
    // compiler testcase.sysy -S -o testcase.s [-O1]
    std::string inputFileName = argv[1];
    if (inputFileName.find(".sy") == std::string::npos) {
        std::cerr << "Error: Input file " << inputFileName << "is not a .sy file\n" << std::endl;
        return 1;
    }
    std::string filePrefix = inputFileName.substr(0, inputFileName.find(".sy"));
    std::string outputFileName;
    bool isOptimize = false;
    for (int i = 1; i < argc; i++) {
        if (std::string(argv[i]) == "-S") {
            if (i + 1 < argc) {
                outputFileName = argv[i + 1];
                i++;
            } else {
                std::cerr << "Error: No output file name\n" << std::endl;
                return 1;
            }
        } else if (std::string(argv[i]) == "-O1") {
            isOptimize = true;
        }
    }

    std::freopen(inputFileName.c_str(), "r", stdin);
    yyparse();
    if (!root) {
        std::cerr << "Error: No program in the FDMJ file\n" << std::endl;
        return 1;
    }
    std::fclose(stdin);

    std::ofstream astOut(filePrefix + ".src");
    ast::Printer printer(astOut, root);
    printer.print();
    astOut.close();

    irpgen::AST2IRP ast2irp(root);
    std::cerr << "Translating AST to IRP..." << std::endl;
    ast2irp.translate();
    std::cerr << "Translation finished" << std::endl;
    irp::Prog *irpProg = ast2irp.result;

    std::ofstream irpOut(filePrefix + ".irp");
    irp::Printer irpPrinter(irpOut, irpProg);
    irpPrinter.print();
    irpOut.close();

    std::ofstream linearOut(filePrefix + ".linear");
    for (auto func : *irpProg->funcs) {
        if (*func->name != "main") {
            continue;
        }
        auto linearizedFunc = canon::linearize(func->body);
        linearOut << "====== Function " << *func->name << " ======" << std::endl;
        irp::printStmtList(linearOut, linearizedFunc, 0);
        linearOut << std::endl;
    }
    linearOut.close();

    return 0;
}